/* Pulsgenerator 
 * 
 * HD44780-kompatibles LCDs im 4-Bit-Modus an PB0, PB3, PB4, PB5, PD4, PD5, PD6, PD7
 *
 *
 * Seitenzahlen gemaess Datenblatt ATmega168 Rev. 2545T-AVR-05/11
 *
 * 06.11.2011 DL1DOW
 */


#ifndef F_CPU
  #define F_CPU 16000000UL
#endif

#include <avr/io.h> 
#include <util/delay.h>
#include <stdio.h>
#include <avr/interrupt.h>
#include "lcd4.h"
#include "glcd_ks0108.h"
#include "glcd_textfun.h"


// Settings ADC
// Referenzspannung: AVcc mit externem, Kondensator an ARef,
// Ergebnis linksbndig ausrichten:
#define ADCMUXCONFIG (1<<REFS0)|(0<<REFS1)|(1 << ADLAR)


// Funktionsdeklarationen
uint8_t getadc(uint8_t muxChannel);

// Variablendeklaration
char strOut[41];        // String fuer Textoutput
int clock = 0;          // Hauptschleifendurchlaufzaehler (Debugging)
uint8_t tOnSoll = 0;    // Pulsbreite   (default)
uint8_t tOffSoll = 255; // Pausenbreite (default)

// Konstanten
#define T_ON_MIN 8      // 4 s, untere Grenze Pulsbreite
#define T_OFF_MIN 8     // 4 s, untere Grenze Pausenbreite




int main(void)
{


    // ADC initialisieren
	while (bit_is_set(ADCSRA, ADSC));
    ADMUX = ADCMUXCONFIG;

    // Datenblatt S.256
    // ADC einschalten
    // 16 MHz / 0,2 MHz = 80 -> Teilungsfaktor 128 // ADC einschalten
    // 16 MHz / 128 = 125 kHz ADC-Clock
    ADCSRA = (1<<ADEN)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);

    // Free running mode.
    //ADCSRB = (0<<ADTS2) | (0<<ADTS1) | (0<<ADTS0);

    // Digitaler Input ADC3 und ADC4 aus (nicht ATmega32)
    //DIDR0 |= (1 << ADC3D);  
    //DIDR0 |= (1 << ADC4D); 
    //DIDR0 |= (1 << ADC5D); 

    // Timer1 einstellen (S.132)
	// PWM-Modus 14 
	// Timer1 an OC1A (PB1)
	// OCR1A : Schaltpunkt 1
	// ICR1  : Schaltpunkt 2
	TCCR1A  = (1<<COM1A1)|(1<<COM1A0)|(0<<COM1B1)|(0<<COM1B0)|(1<<WGM11)|(0<<WGM10);
	TCCR1B = (0<<ICNC1)|(0<<ICES1)|(1<<WGM13)|(1<<WGM12)|(0<<CS12)|(1<<CS11)|(0<<CS10);
    OCR1A = 128;
	ICR1 = 150;
    DDRD |= (1 << PD5);  // PWM-Pin vorbereiten
 	sei(); 
    // Ende ADC


	// Entsprechende Pins auf Ausgang und LCD initialisieren
	lcd_init();                      

	// LCD loeschen
	lcd_clear();  
  
    lcd_putstr("Init OK");

    int sensor1 = 0;



    // Grafik-LCD vorbereiten
	glcd_init(); 

    glcd_pos(2,0);
    glcd_putstr("Hallo");

    glcd_pos(3,0);

    while(1)
       {
	    // Hauptschleife ist sehr langsam, aber zeitunkritisch,
		// da das Timing ueber die PWM-Timer erzeugt wird.

        // Vorgaben aus Potis auslesen und begrenzen
        uint8_t ergebnis1 = getadc(3);
        uint8_t ergebnis2 = getadc(4);
        uint8_t sense1 = getadc(5);

        sensor1 = (7*sensor1 + sense1)/8;

        tOnSoll = ergebnis1/2;
    	if (tOnSoll < T_ON_MIN)
        	 tOnSoll = T_ON_MIN;

        tOffSoll = ergebnis2/2;
        if (tOffSoll < T_OFF_MIN)
	         tOffSoll = T_OFF_MIN;


        // In Timer ablegen
        OCR1A = tOnSoll;
	    ICR1 = tOnSoll+tOffSoll;


        // Display updaten
        lcd_pos(0,0);
        snprintf(strOut,40,"CK=%4d Sen=%3d",clock++,sensor1);
	    lcd_putstr(strOut);
        // In zweite Zeile erstes Zeichen 
	    lcd_pos(0,1);   

        snprintf(strOut,40,"ON=%2d%cs OFF=%2d%cs    ",tOnSoll/2,0xE4,tOffSoll/2,0xE4);
	    lcd_putstr(strOut);


        glcd_pos(1,0);
        //glcd_putstr(strOut); // <- die boese Zeile

        glcd_pos(3,0);
        snprintf(strOut,10,"%4d" ,clock);
        glcd_putstr(strOut);

        }
	return 1;
}


/* ADC-Teile */
// ADC-Kanal auslesen (nur 8 Bit)
// Routine sehr langsam, da warten
// channel: 0 - 6
// Konstante ADCMUXCONFIG
// Seite 225
uint8_t getadc(uint8_t muxChannel)
{
    // Erst ADCL, dann ADCH auslesen. Danach ist das Register geloescht.
    // hier wird nur ADCH benoetigt und ADCL ignoriert:
    uint8_t ergebnis;

    ADMUX = ADCMUXCONFIG | muxChannel;
    ADCSRA  |= (1 << ADSC);   // Wandlung starten
	while (bit_is_set(ADCSRA, ADSC)); // warten
    ergebnis = ADCH; // Wert auslesen und wegwerfen
    ergebnis = ADCH; // obere 8 Bit auslesen

    return ergebnis;
}





