
#include <avr/io.h> 
#include <util/delay.h>
#include "glcd_ks0108.h" 
#include <avr/pgmspace.h>

#define BITMAP_OFFSET 64 // 




void SetPixel(int X,int Y)
{
  X%=64;
  Y%=64;
  glcd_putcom(0b10111000|(X/8)); //X-Adresse-Reihe
  glcd_putcom(0b01000000|Y); //Y-Adresse

  char glcdtemp = 0x00;
  glcdtemp=glcd_readdat();
  glcdtemp=glcd_readdat();
  glcdtemp |=(1<<(X%8));

  asm volatile ("nop");

  glcd_putcom(0b01000000|Y); //Y-Adresse
  glcd_putdat(glcdtemp); 
}



void SetBitmap(int PosX,int PosY,int wX,int wY,const char* zeiger )
{
  PosX%=64;
  PosY%=64;
  for (int j=0; j<wX/8; j++)
  {
    glcd_putcom(0b10111000|(PosX/8+j)); //X-Adresse-Reihe
    glcd_putcom(0b01000000|PosY); //Y-Adresse  
    glcd_wait();

    uint8_t faktor;
    if (wY<=16)
    {
      faktor=4;
    }
    else
    {
      faktor=wY/8;
    }

    for (int k=0; k<wY; k++)
    {
      char h = 0;
      char x = pgm_read_byte(zeiger+faktor*(k)+j+BITMAP_OFFSET);

      // Byte umdrehen
      for(int i = 0; i < 8; i++) 
      {
        h = (h << 1) + (x & 1); 
        x >>= 1; 
      }

      glcd_putdat(~h);
      }
  }

}






char glcd_readdat()
{
  DDR_GLCDDATA = 0x00;
  while(PIN_GLCDDATA & 0x80){
  }; //busy abwarten

  PORT_GLCDCTRL |=  (1<<GLCD_RW);
  PORT_GLCDCTRL |=  (1<<GLCD_RS);
  //delay(1);
  PORT_GLCDCTRL |=  (1<<GLCD_ECLK);
  _delay_ms(1);
  char output = PIN_GLCDDATA;
  PORT_GLCDCTRL &= ~(1<<GLCD_ECLK);    
  PORT_GLCDCTRL &= ~(1<<GLCD_RW);

  while(PIN_GLCDDATA & 0x80){
  }; //busy abwarten
  PORT_GLCDCTRL &= ~(1<<GLCD_RS);

  return output;
} 


/* Sendet ein beliebiges Kommando ans LCD. 
 * Nach dem Schreiben erfolgt der Enable-
 * Puls, nach der kurzen Pause werden alle Pins
 * wieder auf Null gesetzt.
 */
void glcd_putcom(char value) {
  glcd_wait();
  
  DDR_GLCDDATA = 0xFF;
  PORT_GLCDCTRL &= ~(1 << GLCD_RS);
  PORT_GLCDCTRL &= ~(1 << GLCD_RW); 
  PORT_GLCDDATA = value;
  glcd_enablepulse();
  glcd_wait();
  PORT_GLCDDATA = 0x00;
}

/* Sendet beliebige Daten im 8-bit-Modus 
 * ans LCD. Nach dem Schreiben erfolgt der Enable-
 * Puls. 
 * Unterschied zu  glcd_putcom: GLCD_RS ist HIGH
 */
void glcd_putdat(char value) {
  glcd_wait();
  
  DDR_GLCDDATA = 0xFF;
  PORT_GLCDCTRL |=  (1 << GLCD_RS);
  PORT_GLCDCTRL &= ~(1 << GLCD_RW); 
  PORT_GLCDDATA = value;
  glcd_enablepulse();
  glcd_wait();
  PORT_GLCDDATA = 0x00;
}




/* Sendet Enable-Impuls und setzt anschliessend 
 * wieder alle Pins auf Null */
void glcd_enablepulse()
{
  // Enable-Impuls min. 450ns 
  PORT_GLCDCTRL &= ~(1 << GLCD_ECLK);
  asm volatile ("nop");
  asm volatile ("nop");
  asm volatile ("nop");
  asm volatile ("nop");
  asm volatile ("nop");
  asm volatile ("nop");
  asm volatile ("nop");
  asm volatile ("nop");
  PORT_GLCDCTRL |=  (1 << GLCD_ECLK);
  asm volatile ("nop");
  asm volatile ("nop");
  asm volatile ("nop");
  asm volatile ("nop");
  asm volatile ("nop");
  asm volatile ("nop");
  asm volatile ("nop");
  asm volatile ("nop");
  PORT_GLCDCTRL &= ~(1 << GLCD_ECLK);
}

void glcd_right()
{
  PORT_GLCDCTRL |=  (1 << GLCD_CS1);
  PORT_GLCDCTRL &= ~(1 << GLCD_CS2);
  }
  
void glcd_left()
{
  PORT_GLCDCTRL &= ~(1 << GLCD_CS1);
  PORT_GLCDCTRL |=  (1 << GLCD_CS2);
  }  


void glcd_wait()
{
  // Abfragen des Busy-Flags und warten
  // Leider muessen nach dem Loeschen des Busy-Flags
  // noch einige Takte weiter gewartet werden.
  PORT_GLCDDATA = 0x00;             // Pins zuruecksetzen, sonst Pull-Up
  DDR_GLCDDATA  = 0x00;
  PORT_GLCDCTRL &= ~(1 << GLCD_RS);
  PORT_GLCDCTRL |=  (1 << GLCD_RW);
  PORT_GLCDCTRL &= ~(1 << GLCD_RS);
  // Fuer Abfrage Busy-Flag muss mindesten ein CS aktiv sein
  char cs_stat = PORT_GLCDCTRL; 
  PORT_GLCDCTRL |=  (1 << GLCD_CS1);
  PORT_GLCDCTRL &= ~(1 << GLCD_CS2);
  asm volatile ("nop");
  asm volatile ("nop");
  while(PIN_GLCDDATA & 0x80);
  asm volatile ("nop");
  PORT_GLCDCTRL &= ~(1 << GLCD_RW);
  DDR_GLCDDATA  = 0xFF;
  asm volatile ("nop");
  // CS-Status wiederherstellen.
  // Kann ruhig langsam sein, ist eh Warteschleife
  PORT_GLCDCTRL &= ~(1 << GLCD_CS1); 
  PORT_GLCDCTRL &= ~(1 << GLCD_CS2);
  PORT_GLCDCTRL |= (cs_stat & ((1<<GLCD_CS1)|(1<<GLCD_CS2))); // 4 Takte
  asm volatile ("nop");
  asm volatile ("nop");
  asm volatile ("nop");
  asm volatile ("nop");
  asm volatile ("nop");
  asm volatile ("nop");
  asm volatile ("nop"); // keins zuviel!
  }


void glcd_pos(uint8_t PosX,uint8_t PosY)
{
   PosX%=8;                        // erlaubter Bereich
   PosY%=64;                       // erlaubter Bereich
   glcd_putcom(0b10111000|(PosX)); //X-Adresse-Reihe
   glcd_putcom(0b01000000|PosY);   //Y-Adresse  
  }


/* PORT vorbereiten, Display einschalten und loeschen */
void glcd_init()
{
  // Port auf Ausgang
  DDR_GLCDDATA |= 0xFF;
  DDR_GLCDCTRL |= ((1<<GLCD_RS)|(1<<GLCD_RW)|(1<<GLCD_ECLK)|(1<<GLCD_CS1)|(1<<GLCD_CS2)|(1<<GLCD_RESET));

  // Reset kann man sich sparen
  
  // Display einschalten (links und rechts)
  glcd_left();
  glcd_putcom(0b00111111); // Display ON
  glcd_putcom(0b11000000); // Erste Zeile Anfang
  glcd_putcom(0b10111000); // Erste Zeile (X)
  glcd_putcom(0b01000000); // Erste Spalte (Y)

  glcd_right();
  glcd_putcom(0b00111111);
  glcd_putcom(0b11000000); // Erste Zeile
  glcd_putcom(0b10111000); // Erste Zeile (X)
  glcd_putcom(0b01000000); // Erste Spalte (Y)

  glcd_clear();
}


/*
 * Der Inhalt des Displays wird geloescht, indem das ganze
 * Bild mit leeren Pixeln ueberschrieben wird.
 */
void glcd_clear()
{
  // Bilder leeren
  glcd_left();
  for(uint8_t j=0; j<=7; j++)
  {
    glcd_putcom(0b10111000|j); //X-Adresse
    glcd_putcom(0b01000000); //y-Adresse 0 
    for (uint8_t i=0; i<=63; i++) glcd_putdat(0x00); // leere Linie schreiben:
  }
  glcd_putcom(0b11000000); // Erste Zeile
  glcd_putcom(0b10111000); // Erste Zeile (X)
  glcd_putcom(0b01000000); // Erste Spalte (Y)

  glcd_right();
  for(uint8_t j=0; j<=7; j++)
  {
    glcd_putcom(0b10111000|j); //X-Adresse
    glcd_putcom(0b01000000); //y-Adresse 0 
    for(uint8_t i=0; i<=63; i++)  glcd_putdat(0x00); // leere Linie schreiben:
  }

  glcd_putcom(0b11000000); // Erste Zeile
  glcd_putcom(0b10111000); // Erste Zeile (X)
  glcd_putcom(0b01000000); // Erste Spalte (Y)
}


